@@ -55,8 +55,6 @@ class Core(object):
         self.core_que = asyncio.Queue()
         self.backlog = listen
         self.traffic = Traffic()
-        self._ping = asyncio.Event()
-        self._ping.set()
 
     def close(self):
         if not self.f_running:
@@ -74,19 +72,22 @@ class Core(object):
 
     async def ping(self, user: User, f_udp=False):
         try:
-            await asyncio.wait_for(self._ping.wait(), 5.0)
-            self._ping.clear()
+            await asyncio.wait_for(user.event.wait(), 10.0)
+            user.event.clear()
             await self.send_msg_body(msg_body=b'Ping', user=user, allow_udp=f_udp, f_pro_force=True)
-            await asyncio.wait_for(self._ping.wait(), 5.0)
-            self._ping.set()
+            await asyncio.wait_for(user.event.wait(), 5.0)
+            user.event.set()
             return True
         except asyncio.TimeoutError:
             log.debug(f"failed to udp ping {user}")
-            self._ping.set()
+            user.event.set()
             return False
         except ConnectionError as e:
             log.debug(f"socket error on ping by {e}")
-            self._ping.set()
+            user.event.set()
+            return False
+        except Exception:
+            log.error("ping exception", exc_info=True)
             return False
 
     def start(self, s_family=socket.AF_UNSPEC):
@@ -112,6 +113,8 @@ class Core(object):
         }
 
     async def create_connection(self, host, port):
+        if self.f_stop:
+            return False
         for res in socket.getaddrinfo(host, port, socket.AF_UNSPEC, socket.SOCK_STREAM):
             af, socktype, proto, canonname, host_port = res
             if host_port[0] in ban_address:
@@ -125,10 +128,14 @@ class Core(object):
                 else:
                     sock = socket.socket(af, socktype, proto)
                 sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-                sock.connect(host_port)
+                future = loop.run_in_executor(None, sock.connect, host_port)
+                await asyncio.wait_for(future, 10.0)
+                future.result()  # raised exception of socket
                 sock.setblocking(False)
                 reader, writer = await asyncio.open_connection(sock=sock, loop=loop)
                 break
+            except asyncio.TimeoutError:
+                continue  # try to connect but do not reach
             except ConnectionRefusedError:
                 continue  # try to connect closed socket
             except OSError as e:
@@ -236,10 +243,8 @@ class Core(object):
         else:
             return False
 
-    async def send_msg_body(self, msg_body, user: Optional[User] = None, status=200, allow_udp=False, f_pro_force=False):
-        # StatusCode: https://ja.wikipedia.org/wiki/HTTPステータスコード
+    async def send_msg_body(self, msg_body, user: Optional[User] = None, allow_udp=False, f_pro_force=False):
         assert isinstance(msg_body, bytes), 'msg_body is bytes'
-        assert 200 <= status < 600, 'Not found status code {}'.format(status)
 
         # check user existence
         if len(self.user) == 0:
@@ -250,9 +255,9 @@ class Core(object):
 
         # send message
         if allow_udp and f_pro_force:
-            self._send_udp_body(msg_body, user)
+            loop.run_in_executor(None, self.send_udp_body, msg_body, user)
         elif allow_udp and user.header.p2p_udp_accept and len(msg_body) < 1400:
-            self._send_udp_body(msg_body, user)
+            loop.run_in_executor(None, self.send_udp_body, msg_body, user)
         else:
             msg_body = zlib.compress(msg_body)
             msg_body = AESCipher.encrypt(key=user.aeskey, raw=msg_body)
@@ -262,12 +267,14 @@ class Core(object):
             self.traffic.put_traffic_up(send_data)
         return user
 
-    def _send_udp_body(self, msg_body, user):
+    def send_udp_body(self, msg_body, user):
+        """send UDP message"""
         name_len = len(V.SERVER_NAME.encode()).to_bytes(1, 'big')
         msg_body = AESCipher.encrypt(key=user.aeskey, raw=msg_body)
         send_data = name_len + V.SERVER_NAME.encode() + msg_body
         host_port = user.get_host_port()
         sock_family = socket.AF_INET if len(host_port) == 2 else socket.AF_INET6
+        # warning: may block this closure, use run_in_executor
         with socket.socket(sock_family, socket.SOCK_DGRAM) as sock:
             sock.sendto(send_data, host_port)
         self.traffic.put_traffic_up(send_data)
@@ -444,7 +451,7 @@ class Core(object):
                     await self.send_msg_body(b'Pong', user)
                 elif msg_body == b'Pong':
                     log.debug(f"receive Pong from {user.header.name}")
-                    self._ping.set()
+                    user.event.set()
                 else:
                     await self.core_que.put((user, msg_body))
                 f_raise_timeout = False
@@ -483,9 +490,12 @@ class Core(object):
             host_port = new_user.get_host_port()
             af = socket.AF_INET if len(host_port) == 2 else socket.AF_INET6
             sock = socket.socket(af, socket.SOCK_STREAM)
+            sock.settimeout(5.0)
             try:
-                r = sock.connect_ex(host_port)
-                if r != 0:
+                future = loop.run_in_executor(None, sock.connect_ex, host_port)
+                await asyncio.wait_for(future, 60.0)
+                result = future.result()
+                if result != 0:
                     f_tcp = False
                 sock.close()
             except OSError:
@@ -563,20 +573,20 @@ async def udp_server_handle(msg, addr, core: Core):
         log.debug("UDP handle exception", exc_info=Debug.P_PRINT_EXCEPTION)
 
 
-def create_tcp_server(core: Core, family):
+def create_tcp_server(core: Core, family, host_port):
     assert family == socket.AF_INET or family == socket.AF_INET6
     coroutine = asyncio.start_server(
-        core.initial_connection_check, core.host, V.P2P_PORT,
+        core.initial_connection_check, host_port[0], host_port[1],
         family=family, backlog=core.backlog, loop=loop)
     return loop.run_until_complete(coroutine)
 
 
-def create_udp_server(core: Core, family):
+def create_udp_server(core: Core, family, host_port):
     assert family == socket.AF_INET or family == socket.AF_INET6
     sock = socket.socket(family, socket.SOCK_DGRAM, 0)
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     sock.setblocking(False)
-    sock.bind((core.host, V.P2P_PORT))
+    sock.bind(host_port)
     fd = sock.fileno()
 
     def listen():
@@ -600,7 +610,7 @@ def setup_all_socket_server(core: Core, s_family):
         for res in socket.getaddrinfo(core.host, V.P2P_PORT, s_family, socket.SOCK_STREAM, 0, socket.AI_PASSIVE):
             af, sock_type, proto, canon_name, sa = res
             try:
-                sock = create_tcp_server(core, af)
+                sock = create_tcp_server(core, af, sa)
                 log.debug(f"success tcp server creation af={socket2name.get(af)}")
                 tcp_servers.append(sock)
                 V.P2P_ACCEPT = True
@@ -612,7 +622,7 @@ def setup_all_socket_server(core: Core, s_family):
         for res in socket.getaddrinfo(core.host, V.P2P_PORT, s_family, socket.SOCK_DGRAM, 0, socket.AI_PASSIVE):
             af, sock_type, proto, canon_name, sa = res
             try:
-                sock = create_udp_server(core, af)
+                sock = create_udp_server(core, af, sa)
                 log.debug(f"success udp server creation af={socket2name.get(af)}")
                 udp_servers.append(sock)
                 V.P2P_UDP_ACCEPT = True
