@@ -66,9 +66,10 @@ class Peer2Peer(object):
         self.core.close()
 
     def setup(self, s_family=socket.AF_UNSPEC, f_stabilize=True):
+        broadcast_que = asyncio.Queue()
+
         async def inner_loop():
             user: Optional[User] = None
-            broadcast_task: Optional[asyncio.Future] = None
             log.info("start P2P inner loop")
             while not self.f_stop:
                 try:
@@ -88,12 +89,8 @@ class Peer2Peer(object):
                         log.debug("unrecognized message receive")
                     elif item['type'] == T_REQUEST:
                         if item['cmd'] == Peer2PeerCmd.BROADCAST:
-                            # broadcast cmd is another thread
-                            if broadcast_task:
-                                if not broadcast_task.done():
-                                    await asyncio.wait_for(broadcast_task, 10.0)
-                            # overwrite new task
-                            broadcast_task = asyncio.ensure_future(self.type_request(user, item))
+                            # process broadcast cmd one by one
+                            await broadcast_que.put((user, item, time()))
                         else:
                             await self.type_request(user, item)
                     elif item['type'] == T_RESPONSE:
@@ -105,19 +102,39 @@ class Peer2Peer(object):
                         log.debug(f"unknown type={item['type']}")
                 except asyncio.TimeoutError:
                     log.warning(f"timeout on broadcast and cancel task")
-                    broadcast_task.cancel()
+                    broadcast_task = None
                 except Exception:
                     log.debug(f"core que processing exception of {user}", exc_info=True)
             self.f_finish = True
             self.f_running = False
             log.info("close inner_loop process")
 
+        async def broadcast_loop():
+            log.info("start broadcast_loop")
+            while not self.f_stop:
+                try:
+                    user, item, ntime = await asyncio.wait_for(broadcast_que.get(), 1.0)
+                except asyncio.TimeoutError:
+                    continue
+                try:
+                    if time() - ntime < 5.0:
+                        await asyncio.wait_for(self.type_request(user, item), 10.0)
+                    else:
+                        log.warning(f"try to process broadcast but too late, {int(time()-ntime)}s")
+                except asyncio.TimeoutError:
+                    log.info(f"broadcast relay failed from {user}")
+                    user.warn += 1
+                except Exception:
+                    log.warning('broadcast_loop exception', exc_info=True)
+            log.info("close broadcast_loop")
+
         assert not loop.is_running(), "setup before event loop start!"
         self.core.start(s_family=s_family)
         if f_stabilize:
             loop_futures.append(asyncio.ensure_future(auto_stabilize_network(self)))
         # Processing
         loop_futures.append(asyncio.ensure_future(inner_loop()))
+        loop_futures.append(asyncio.ensure_future(broadcast_loop()))
         log.info(f"start user, name={V.SERVER_NAME} port={V.P2P_PORT}")
         self.f_running = True
 
@@ -183,7 +200,7 @@ class Peer2Peer(object):
                 port = item['data']['port']
             except Exception as e:
                 port = user.header.p2p_port
-            temperate['data'] = is_reachable(host=user.host_port[0], port=port)
+            temperate['data'] = await is_reachable(host=user.host_port[0], port=port)
             allows.append(user)
 
         elif item['cmd'] == Peer2PeerCmd.DIRECT_CMD:
@@ -220,6 +237,10 @@ class Peer2Peer(object):
             future = self._result_ques[uuid]
             if not future.done():
                 future.set_result((user, data))
+            else:
+                log.debug(f"uuid={uuid} type_response failed, already future done? {future}")
+        else:
+            log.debug(f"uuid={uuid} type_response failed, not found uuid")
 
     async def type_ack(self, user: User, item: dict):
         # cmd = item['cmd']
@@ -283,17 +304,20 @@ class Peer2Peer(object):
         try:
             await asyncio.wait_for(future, timeout)
             receive_user, item = future.result()
-            receive_user.warn = 0
             return receive_user, item
-        except asyncio.TimeoutError:
-            future.cancel()
-            if user:
-                if 3 < user.warn:
-                    await self.try_reconnect(user, reason="too many warn point")
-                else:
-                    user.warn += 1
-            log.debug(f"timeout on sending cmd({cmd}) to {user}, id={uuid}")
-        raise asyncio.TimeoutError(f"timeout cmd")
+        except (asyncio.TimeoutError, asyncio.CancelledError):
+            # timeout set CancelledError exception to future
+            # do not care future
+            pass
+        except Exception:
+            log.debug("send_command exception", exc_info=True)
+        if user:
+            if await self.core.ping(user):
+                log.debug("timeout but ping success")
+            else:
+                await self.try_reconnect(user, reason="ping failed on send_command")
+        log.debug(f"timeout on sending cmd({cmd}) to {user}, id={uuid}")
+        raise asyncio.TimeoutError("timeout cmd")
 
     async def try_reconnect(self, user, reason):
         self.core.remove_connection(user, reason)
@@ -474,13 +498,11 @@ async def auto_stabilize_network(p2p: Peer2Peer):
                         del user_score[host_port]
             else:
                 # check warning point too high user
-                for user in sorted(p2p.core.user, key=lambda x: x.warn, reverse=True):
-                    if user.warn < 5:
-                        continue
-                    if await p2p.core.ping(user, f_udp=False):
+                for user in p2p.core.user.copy():
+                    if await p2p.core.ping(user):
                         continue
                     # looks problem on this user
-                    p2p.core.remove_connection(user, 'too many warn and ping failed')
+                    p2p.core.remove_connection(user, 'ping failed on stabilize loop')
                     break
 
         except asyncio.TimeoutError as e:
